'use strict';

exports.__esModule = true;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var LINE_WIDTH_FACTOR = 20;

var FONT_SIZE_FACTOR = 2.5;
var STAR_POINT_SIZE_FACTOR = 8;
var BACKGROUND_COLOR = 'rgb(251, 196, 103)';

// BoardDrawer exposes functions for drawing a go board on a 2d context

var BoardDrawer = function () {
  function BoardDrawer(boardSize, width, height) {
    _classCallCheck(this, BoardDrawer);

    this.boxSize = width / (boardSize + 1);
    this.lineWidth = this.boxSize / LINE_WIDTH_FACTOR;
    this.fontSize = this.boxSize / FONT_SIZE_FACTOR;
    this.width = width;
    this.height = height;
    this.boardSize = boardSize;
    this.stars = this.calculateStars();
  }

  BoardDrawer.prototype.calculateStars = function calculateStars() {
    var boardSize = this.boardSize,
        boxSize = this.boxSize;

    var starPos = boardSize === 9 ? 3 : 4;
    var single = starPos * boxSize;
    var double = (boardSize + 1) / 2 * boxSize;
    var triple = (boardSize + 1 - starPos) * boxSize;
    var stars = [{ x: double, y: double }];

    if (boardSize % 2 !== 0 && boardSize > 7) {
      stars.push(
      // TopLeft
      { x: single, y: single },
      // Left
      { x: single, y: double },
      // BottomLeft
      { x: single, y: triple },
      // BottomMiddle
      { x: double, y: triple },
      // TopMiddle
      { x: double, y: single },
      // TopRight
      { x: triple, y: single },
      // MiddleRight
      { x: triple, y: double },
      // BottomRight
      { x: triple, y: triple });
    }

    return stars;
  };

  BoardDrawer.prototype.clear = function clear(ctx) {
    var width = this.width,
        height = this.height;


    ctx.save();
    ctx.clearRect(0, 0, width, height);
    ctx.fillStyle = BACKGROUND_COLOR;
    ctx.fillRect(0, 0, width, height);
    ctx.restore();
  };

  BoardDrawer.prototype.stones = function stones(ctx, moves) {
    var lineWidth = this.lineWidth,
        boxSize = this.boxSize;


    ctx.save();
    ctx.lineWidth = lineWidth * 2;

    moves.forEach(function (move, index) {
      var isLastMove = index === moves.length - 1;

      // Draw stone
      ctx.fillStyle = move.color === 'B' ? '#000' : '#FFF';
      ctx.beginPath();
      ctx.arc(move.pos[0] * boxSize, move.pos[1] * boxSize, boxSize / 2, 0, 2 * Math.PI);
      ctx.stroke();
      ctx.fill();

      // Mark last move
      if (isLastMove) {
        ctx.beginPath();
        ctx.fillStyle = move.color === 'B' ? '#FFF' : '#000';
        ctx.lineWidth = lineWidth;
        ctx.arc(move.pos[0] * boxSize, move.pos[1] * boxSize, boxSize / 3.5, 0, 2 * Math.PI);
        ctx.stroke();
      }
    });

    ctx.restore();
  };

  BoardDrawer.prototype.grid = function grid(ctx) {
    var lineWidth = this.lineWidth,
        boxSize = this.boxSize,
        width = this.width,
        height = this.height,
        boardSize = this.boardSize;


    ctx.save();
    ctx.beginPath();
    ctx.lineWidth = lineWidth;

    for (var i = 1; i <= boardSize; i++) {
      ctx.moveTo(i * boxSize, boxSize);
      ctx.lineTo(i * boxSize, height - boxSize);
      ctx.moveTo(boxSize, i * boxSize);
      ctx.lineTo(width - boxSize, i * boxSize);
    }

    ctx.stroke();
    ctx.restore();
  };

  BoardDrawer.prototype.starPoints = function starPoints(ctx) {
    var boxSize = this.boxSize,
        boardSize = this.boardSize,
        stars = this.stars;


    ctx.save();
    ctx.beginPath();
    ctx.fillStyle = '#000';

    stars.forEach(function (pos) {
      ctx.moveTo(pos.x, pos.y);
      ctx.arc(pos.x, pos.y, boxSize / STAR_POINT_SIZE_FACTOR, 0, 2 * Math.PI);
    });

    ctx.fill();
    ctx.restore();
  };

  BoardDrawer.prototype.coordinates = function coordinates(ctx) {
    var boxSize = this.boxSize,
        fontSize = this.fontSize,
        boardSize = this.boardSize;


    ctx.save();
    ctx.font = 'bold ' + fontSize + 'px sans-serif';
    ctx.textAlign = 'center';
    ctx.fillStyle = '#000';

    for (var i = 1; i <= boardSize; i++) {
      ctx.fillText(String.fromCharCode(64 + i), i * boxSize, boxSize / 2);
      ctx.fillText('' + (boardSize - i + 1), boxSize / 2, i * boxSize + 3);
    }

    ctx.restore();
  };

  BoardDrawer.prototype.draw = function draw(ctx, moves) {
    this.clear(ctx);
    this.grid(ctx);
    this.coordinates(ctx);
    this.starPoints(ctx);
    this.stones(ctx, moves);
  };

  return BoardDrawer;
}();

exports.default = BoardDrawer;